import { URL, urlToHttpOptions } from 'node:url'
import https from 'node:https'

export const Method = Object.freeze({
    GET:     'GET',
    HEAD:    'HEAD',
    POST:    'POST',
    PUT:     'PUT',
    DELETE:  'DELETE',
    CONNECT: 'CONNECT',
    OPTIONS: 'OPTIONS',
    TRACE:   'TRACE',
    PATCH:   'PATCH'
})

const Response = data => Object.freeze({
    buffer: () => Promise.resolve(data),
    text: () => new Promise((resolve, reject) => {
            try {
                resolve(data.toString())
            } catch(e) {
                reject(e)
            }
    }),
    json: () => new Promise((resolve, reject) => {
        try {
            resolve(JSON.parse(data))
        } catch(e) {
            reject(e)
        }
    })
})

export const request = (path, options) => new Promise((resolve, reject) => {
    const url = new URL(path)

    const opts = {
        ...urlToHttpOptions(url),
        ...options
    }

    const req = https.request(opts, res => {
        let data = []

        res.on('data', d => {
            data.push(d)
        })

        res.on('end', () => resolve(Response(Buffer.concat(data))))
    })
    req.on('error', reject)
    req.end()
})

