import { path } from 'rambda'

const Linker = ({ target, linker, rustflags }) => Object.freeze({ target, linker, rustflags })

const linuxLld = () => Linker({
    linker: 'clang',
    rustflags: ['-C', 'link-arg=-fuse-ld=lld']
})

const macosLld = () => Linker({
    rustflags: [ '-C', 'link-arg=-fuse-ld=/opt/homebrew/opt/llvm/bin/ld64.lld']
})

const windowsLld = () => Linker({
    linker: 'rust-lld.exe',
    rustflags: []
})

const linuxMold = () => Linker({
    linker: 'clang',
    rustflags: ['-C', 'link-arg=-fuse-ld=/usr/bin/mold']

})

export const LinkerType = Object.freeze({
    lld: Object.freeze({
        'x86_64-pc-windows-msvc': windowsLld,
        'aarch64-apple-darwin': macosLld,
        'x86_64-unknown-linux-gnu': linuxLld
    }),
    mold: Object.freeze({
        'x86_64-unknown-linux-gnu': linuxMold
    })
})

export const get = (linker, host)=> {
    const type = path([linker.toLowerCase(), host], LinkerType)

    if(type) {
        return type()
    } else {
        return undefined
    }
}
